# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Literal

import httpx

from ..types import balance_retrieve_ledger_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultPageNumberPagination, AsyncDefaultPageNumberPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.balance_ledger_entry import BalanceLedgerEntry

__all__ = ["BalancesResource", "AsyncBalancesResource"]


class BalancesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> BalancesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#accessing-raw-response-data-eg-headers
        """
        return BalancesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BalancesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#with_streaming_response
        """
        return BalancesResourceWithStreamingResponse(self)

    def retrieve_ledger(
        self,
        *,
        created_at_gte: Union[str, datetime] | Omit = omit,
        created_at_lte: Union[str, datetime] | Omit = omit,
        currency: Literal[
            "AED",
            "ALL",
            "AMD",
            "ANG",
            "AOA",
            "ARS",
            "AUD",
            "AWG",
            "AZN",
            "BAM",
            "BBD",
            "BDT",
            "BGN",
            "BHD",
            "BIF",
            "BMD",
            "BND",
            "BOB",
            "BRL",
            "BSD",
            "BWP",
            "BYN",
            "BZD",
            "CAD",
            "CHF",
            "CLP",
            "CNY",
            "COP",
            "CRC",
            "CUP",
            "CVE",
            "CZK",
            "DJF",
            "DKK",
            "DOP",
            "DZD",
            "EGP",
            "ETB",
            "EUR",
            "FJD",
            "FKP",
            "GBP",
            "GEL",
            "GHS",
            "GIP",
            "GMD",
            "GNF",
            "GTQ",
            "GYD",
            "HKD",
            "HNL",
            "HRK",
            "HTG",
            "HUF",
            "IDR",
            "ILS",
            "INR",
            "IQD",
            "JMD",
            "JOD",
            "JPY",
            "KES",
            "KGS",
            "KHR",
            "KMF",
            "KRW",
            "KWD",
            "KYD",
            "KZT",
            "LAK",
            "LBP",
            "LKR",
            "LRD",
            "LSL",
            "LYD",
            "MAD",
            "MDL",
            "MGA",
            "MKD",
            "MMK",
            "MNT",
            "MOP",
            "MRU",
            "MUR",
            "MVR",
            "MWK",
            "MXN",
            "MYR",
            "MZN",
            "NAD",
            "NGN",
            "NIO",
            "NOK",
            "NPR",
            "NZD",
            "OMR",
            "PAB",
            "PEN",
            "PGK",
            "PHP",
            "PKR",
            "PLN",
            "PYG",
            "QAR",
            "RON",
            "RSD",
            "RUB",
            "RWF",
            "SAR",
            "SBD",
            "SCR",
            "SEK",
            "SGD",
            "SHP",
            "SLE",
            "SLL",
            "SOS",
            "SRD",
            "SSP",
            "STN",
            "SVC",
            "SZL",
            "THB",
            "TND",
            "TOP",
            "TRY",
            "TTD",
            "TWD",
            "TZS",
            "UAH",
            "UGX",
            "USD",
            "UYU",
            "UZS",
            "VES",
            "VND",
            "VUV",
            "WST",
            "XAF",
            "XCD",
            "XOF",
            "XPF",
            "YER",
            "ZAR",
            "ZMW",
        ]
        | Omit = omit,
        event_type: Literal[
            "payment",
            "refund",
            "refund_reversal",
            "dispute",
            "dispute_reversal",
            "tax",
            "tax_reversal",
            "payment_fees",
            "refund_fees",
            "refund_fees_reversal",
            "dispute_fees",
            "payout",
            "payout_fees",
            "payout_reversal",
            "payout_fees_reversal",
            "dodo_credits",
            "adjustment",
            "currency_conversion",
        ]
        | Omit = omit,
        limit: int | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        reference_object_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultPageNumberPagination[BalanceLedgerEntry]:
        """
        Args:
          created_at_gte: Get events after this created time

          created_at_lte: Get events created before this time

          currency: Filter by currency

          event_type: Filter by Ledger Event Type

          limit: Min : 1, Max : 100, default 10

          page_number: Page number default is 0

          page_size: Page size default is 10 max is 100

          reference_object_id: Get events history of a specific object like payment/subscription/refund/dispute

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/balances/ledger",
            page=SyncDefaultPageNumberPagination[BalanceLedgerEntry],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "created_at_gte": created_at_gte,
                        "created_at_lte": created_at_lte,
                        "currency": currency,
                        "event_type": event_type,
                        "limit": limit,
                        "page_number": page_number,
                        "page_size": page_size,
                        "reference_object_id": reference_object_id,
                    },
                    balance_retrieve_ledger_params.BalanceRetrieveLedgerParams,
                ),
            ),
            model=BalanceLedgerEntry,
        )


class AsyncBalancesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncBalancesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#accessing-raw-response-data-eg-headers
        """
        return AsyncBalancesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBalancesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#with_streaming_response
        """
        return AsyncBalancesResourceWithStreamingResponse(self)

    def retrieve_ledger(
        self,
        *,
        created_at_gte: Union[str, datetime] | Omit = omit,
        created_at_lte: Union[str, datetime] | Omit = omit,
        currency: Literal[
            "AED",
            "ALL",
            "AMD",
            "ANG",
            "AOA",
            "ARS",
            "AUD",
            "AWG",
            "AZN",
            "BAM",
            "BBD",
            "BDT",
            "BGN",
            "BHD",
            "BIF",
            "BMD",
            "BND",
            "BOB",
            "BRL",
            "BSD",
            "BWP",
            "BYN",
            "BZD",
            "CAD",
            "CHF",
            "CLP",
            "CNY",
            "COP",
            "CRC",
            "CUP",
            "CVE",
            "CZK",
            "DJF",
            "DKK",
            "DOP",
            "DZD",
            "EGP",
            "ETB",
            "EUR",
            "FJD",
            "FKP",
            "GBP",
            "GEL",
            "GHS",
            "GIP",
            "GMD",
            "GNF",
            "GTQ",
            "GYD",
            "HKD",
            "HNL",
            "HRK",
            "HTG",
            "HUF",
            "IDR",
            "ILS",
            "INR",
            "IQD",
            "JMD",
            "JOD",
            "JPY",
            "KES",
            "KGS",
            "KHR",
            "KMF",
            "KRW",
            "KWD",
            "KYD",
            "KZT",
            "LAK",
            "LBP",
            "LKR",
            "LRD",
            "LSL",
            "LYD",
            "MAD",
            "MDL",
            "MGA",
            "MKD",
            "MMK",
            "MNT",
            "MOP",
            "MRU",
            "MUR",
            "MVR",
            "MWK",
            "MXN",
            "MYR",
            "MZN",
            "NAD",
            "NGN",
            "NIO",
            "NOK",
            "NPR",
            "NZD",
            "OMR",
            "PAB",
            "PEN",
            "PGK",
            "PHP",
            "PKR",
            "PLN",
            "PYG",
            "QAR",
            "RON",
            "RSD",
            "RUB",
            "RWF",
            "SAR",
            "SBD",
            "SCR",
            "SEK",
            "SGD",
            "SHP",
            "SLE",
            "SLL",
            "SOS",
            "SRD",
            "SSP",
            "STN",
            "SVC",
            "SZL",
            "THB",
            "TND",
            "TOP",
            "TRY",
            "TTD",
            "TWD",
            "TZS",
            "UAH",
            "UGX",
            "USD",
            "UYU",
            "UZS",
            "VES",
            "VND",
            "VUV",
            "WST",
            "XAF",
            "XCD",
            "XOF",
            "XPF",
            "YER",
            "ZAR",
            "ZMW",
        ]
        | Omit = omit,
        event_type: Literal[
            "payment",
            "refund",
            "refund_reversal",
            "dispute",
            "dispute_reversal",
            "tax",
            "tax_reversal",
            "payment_fees",
            "refund_fees",
            "refund_fees_reversal",
            "dispute_fees",
            "payout",
            "payout_fees",
            "payout_reversal",
            "payout_fees_reversal",
            "dodo_credits",
            "adjustment",
            "currency_conversion",
        ]
        | Omit = omit,
        limit: int | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        reference_object_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[BalanceLedgerEntry, AsyncDefaultPageNumberPagination[BalanceLedgerEntry]]:
        """
        Args:
          created_at_gte: Get events after this created time

          created_at_lte: Get events created before this time

          currency: Filter by currency

          event_type: Filter by Ledger Event Type

          limit: Min : 1, Max : 100, default 10

          page_number: Page number default is 0

          page_size: Page size default is 10 max is 100

          reference_object_id: Get events history of a specific object like payment/subscription/refund/dispute

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/balances/ledger",
            page=AsyncDefaultPageNumberPagination[BalanceLedgerEntry],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "created_at_gte": created_at_gte,
                        "created_at_lte": created_at_lte,
                        "currency": currency,
                        "event_type": event_type,
                        "limit": limit,
                        "page_number": page_number,
                        "page_size": page_size,
                        "reference_object_id": reference_object_id,
                    },
                    balance_retrieve_ledger_params.BalanceRetrieveLedgerParams,
                ),
            ),
            model=BalanceLedgerEntry,
        )


class BalancesResourceWithRawResponse:
    def __init__(self, balances: BalancesResource) -> None:
        self._balances = balances

        self.retrieve_ledger = to_raw_response_wrapper(
            balances.retrieve_ledger,
        )


class AsyncBalancesResourceWithRawResponse:
    def __init__(self, balances: AsyncBalancesResource) -> None:
        self._balances = balances

        self.retrieve_ledger = async_to_raw_response_wrapper(
            balances.retrieve_ledger,
        )


class BalancesResourceWithStreamingResponse:
    def __init__(self, balances: BalancesResource) -> None:
        self._balances = balances

        self.retrieve_ledger = to_streamed_response_wrapper(
            balances.retrieve_ledger,
        )


class AsyncBalancesResourceWithStreamingResponse:
    def __init__(self, balances: AsyncBalancesResource) -> None:
        self._balances = balances

        self.retrieve_ledger = async_to_streamed_response_wrapper(
            balances.retrieve_ledger,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import Required, TypedDict

__all__ = ["CustomerCreateParams"]


class CustomerCreateParams(TypedDict, total=False):
    email: Required[str]

    name: Required[str]

    metadata: Dict[str, str]
    """Additional metadata for the customer"""

    phone_number: Optional[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import Required, TypedDict

__all__ = ["ShortLinkCreateParams"]


class ShortLinkCreateParams(TypedDict, total=False):
    slug: Required[str]
    """Slug for the short link."""

    static_checkout_params: Optional[Dict[str, str]]
    """Static Checkout URL parameters to apply to the resulting short URL."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel

__all__ = ["SubscriptionChargeResponse"]


class SubscriptionChargeResponse(BaseModel):
    payment_id: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

__all__ = ["AddMeterToPriceParam"]


class AddMeterToPriceParam(TypedDict, total=False):
    meter_id: Required[str]

    credit_entitlement_id: Optional[str]
    """Optional credit entitlement ID to link this meter to for credit-based billing"""

    description: Optional[str]
    """Meter description. Will ignored on Request, but will be shown in response"""

    free_threshold: Optional[int]

    measurement_unit: Optional[str]
    """Meter measurement unit. Will ignored on Request, but will be shown in response"""

    meter_units_per_credit: Optional[str]
    """Number of meter units that equal one credit.

    Required when credit_entitlement_id is set.
    """

    name: Optional[str]
    """Meter name. Will ignored on Request, but will be shown in response"""

    price_per_unit: Optional[str]
    """The price per unit in lowest denomination.

    Must be greater than zero. Supports up to 5 digits before decimal point and 12
    decimal places.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from datetime import datetime

from .price import Price
from .._models import BaseModel
from .currency import Currency
from .tax_category import TaxCategory

__all__ = ["ProductListResponse"]


class ProductListResponse(BaseModel):
    business_id: str
    """Unique identifier for the business to which the product belongs."""

    created_at: datetime
    """Timestamp when the product was created."""

    is_recurring: bool
    """Indicates if the product is recurring (e.g., subscriptions)."""

    metadata: Dict[str, str]
    """Additional custom data associated with the product"""

    product_id: str
    """Unique identifier for the product."""

    tax_category: TaxCategory
    """Tax category associated with the product."""

    updated_at: datetime
    """Timestamp when the product was last updated."""

    currency: Optional[Currency] = None
    """Currency of the price"""

    description: Optional[str] = None
    """Description of the product, optional."""

    image: Optional[str] = None
    """URL of the product image, optional."""

    name: Optional[str] = None
    """Name of the product, optional."""

    price: Optional[int] = None
    """Price of the product, optional.

    The price is represented in the lowest denomination of the currency. For
    example:

    - In USD, a price of `$12.34` would be represented as `1234` (cents).
    - In JPY, a price of `¥1500` would be represented as `1500` (yen).
    - In INR, a price of `₹1234.56` would be represented as `123456` (paise).

    This ensures precision and avoids floating-point rounding errors.
    """

    price_detail: Optional[Price] = None
    """Details of the price"""

    tax_inclusive: Optional[bool] = None
    """Indicates if the price is tax inclusive"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

from .currency import Currency
from .time_interval import TimeInterval
from .cbb_overage_behavior import CbbOverageBehavior

__all__ = ["CreditEntitlementCreateParams"]


class CreditEntitlementCreateParams(TypedDict, total=False):
    name: Required[str]
    """Name of the credit entitlement"""

    overage_enabled: Required[bool]
    """Whether overage charges are enabled when credits run out"""

    precision: Required[int]
    """Precision for credit amounts (0-10 decimal places)"""

    rollover_enabled: Required[bool]
    """Whether rollover is enabled for unused credits"""

    unit: Required[str]
    """Unit of measurement for the credit (e.g., "API Calls", "Tokens", "Credits")"""

    currency: Optional[Currency]
    """Currency for pricing (required if price_per_unit is set)"""

    description: Optional[str]
    """Optional description of the credit entitlement"""

    expires_after_days: Optional[int]
    """Number of days after which credits expire (optional)"""

    max_rollover_count: Optional[int]
    """Maximum number of times credits can be rolled over"""

    overage_behavior: Optional[CbbOverageBehavior]
    """
    Controls how overage is handled at billing cycle end. Defaults to
    forgive_at_reset if not specified.
    """

    overage_limit: Optional[int]
    """Maximum overage units allowed (optional)"""

    price_per_unit: Optional[str]
    """Price per credit unit"""

    rollover_percentage: Optional[int]
    """Percentage of unused credits that can rollover (0-100)"""

    rollover_timeframe_count: Optional[int]
    """Count of timeframe periods for rollover limit"""

    rollover_timeframe_interval: Optional[TimeInterval]
    """Interval type for rollover timeframe"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import TYPE_CHECKING, Any, Dict, Mapping, cast
from typing_extensions import Self, Literal, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    Omit,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
    not_given,
)
from ._utils import is_given, get_async_library
from ._compat import cached_property
from ._version import __version__
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import APIStatusError, DodoPaymentsError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)

if TYPE_CHECKING:
    from .resources import (
        misc,
        addons,
        brands,
        meters,
        payouts,
        refunds,
        balances,
        disputes,
        invoices,
        licenses,
        payments,
        products,
        webhooks,
        customers,
        discounts,
        license_keys,
        usage_events,
        subscriptions,
        checkout_sessions,
        credit_entitlements,
        license_key_instances,
    )
    from .resources.misc import MiscResource, AsyncMiscResource
    from .resources.addons import AddonsResource, AsyncAddonsResource
    from .resources.brands import BrandsResource, AsyncBrandsResource
    from .resources.meters import MetersResource, AsyncMetersResource
    from .resources.payouts import PayoutsResource, AsyncPayoutsResource
    from .resources.refunds import RefundsResource, AsyncRefundsResource
    from .resources.balances import BalancesResource, AsyncBalancesResource
    from .resources.disputes import DisputesResource, AsyncDisputesResource
    from .resources.licenses import LicensesResource, AsyncLicensesResource
    from .resources.payments import PaymentsResource, AsyncPaymentsResource
    from .resources.discounts import DiscountsResource, AsyncDiscountsResource
    from .resources.license_keys import LicenseKeysResource, AsyncLicenseKeysResource
    from .resources.usage_events import UsageEventsResource, AsyncUsageEventsResource
    from .resources.subscriptions import SubscriptionsResource, AsyncSubscriptionsResource
    from .resources.checkout_sessions import CheckoutSessionsResource, AsyncCheckoutSessionsResource
    from .resources.invoices.invoices import InvoicesResource, AsyncInvoicesResource
    from .resources.products.products import ProductsResource, AsyncProductsResource
    from .resources.webhooks.webhooks import WebhooksResource, AsyncWebhooksResource
    from .resources.customers.customers import CustomersResource, AsyncCustomersResource
    from .resources.license_key_instances import LicenseKeyInstancesResource, AsyncLicenseKeyInstancesResource
    from .resources.credit_entitlements.credit_entitlements import (
        CreditEntitlementsResource,
        AsyncCreditEntitlementsResource,
    )

__all__ = [
    "ENVIRONMENTS",
    "Timeout",
    "Transport",
    "ProxiesTypes",
    "RequestOptions",
    "DodoPayments",
    "AsyncDodoPayments",
    "Client",
    "AsyncClient",
]

ENVIRONMENTS: Dict[str, str] = {
    "live_mode": "https://live.dodopayments.com",
    "test_mode": "https://test.dodopayments.com",
}


class DodoPayments(SyncAPIClient):
    # client options
    bearer_token: str
    webhook_key: str | None

    _environment: Literal["live_mode", "test_mode"] | NotGiven

    def __init__(
        self,
        *,
        bearer_token: str | None = None,
        webhook_key: str | None = None,
        environment: Literal["live_mode", "test_mode"] | NotGiven = not_given,
        base_url: str | httpx.URL | None | NotGiven = not_given,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous DodoPayments client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `bearer_token` from `DODO_PAYMENTS_API_KEY`
        - `webhook_key` from `DODO_PAYMENTS_WEBHOOK_KEY`
        """
        if bearer_token is None:
            bearer_token = os.environ.get("DODO_PAYMENTS_API_KEY")
        if bearer_token is None:
            raise DodoPaymentsError(
                "The bearer_token client option must be set either by passing bearer_token to the client or by setting the DODO_PAYMENTS_API_KEY environment variable"
            )
        self.bearer_token = bearer_token

        if webhook_key is None:
            webhook_key = os.environ.get("DODO_PAYMENTS_WEBHOOK_KEY")
        self.webhook_key = webhook_key

        self._environment = environment

        base_url_env = os.environ.get("DODO_PAYMENTS_BASE_URL")
        if is_given(base_url) and base_url is not None:
            # cast required because mypy doesn't understand the type narrowing
            base_url = cast("str | httpx.URL", base_url)  # pyright: ignore[reportUnnecessaryCast]
        elif is_given(environment):
            if base_url_env and base_url is not None:
                raise ValueError(
                    "Ambiguous URL; The `DODO_PAYMENTS_BASE_URL` env var and the `environment` argument are given. If you want to use the environment, you must pass base_url=None",
                )

            try:
                base_url = ENVIRONMENTS[environment]
            except KeyError as exc:
                raise ValueError(f"Unknown environment: {environment}") from exc
        elif base_url_env is not None:
            base_url = base_url_env
        else:
            self._environment = environment = "live_mode"

            try:
                base_url = ENVIRONMENTS[environment]
            except KeyError as exc:
                raise ValueError(f"Unknown environment: {environment}") from exc

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def checkout_sessions(self) -> CheckoutSessionsResource:
        from .resources.checkout_sessions import CheckoutSessionsResource

        return CheckoutSessionsResource(self)

    @cached_property
    def payments(self) -> PaymentsResource:
        from .resources.payments import PaymentsResource

        return PaymentsResource(self)

    @cached_property
    def subscriptions(self) -> SubscriptionsResource:
        from .resources.subscriptions import SubscriptionsResource

        return SubscriptionsResource(self)

    @cached_property
    def invoices(self) -> InvoicesResource:
        from .resources.invoices import InvoicesResource

        return InvoicesResource(self)

    @cached_property
    def licenses(self) -> LicensesResource:
        from .resources.licenses import LicensesResource

        return LicensesResource(self)

    @cached_property
    def license_keys(self) -> LicenseKeysResource:
        from .resources.license_keys import LicenseKeysResource

        return LicenseKeysResource(self)

    @cached_property
    def license_key_instances(self) -> LicenseKeyInstancesResource:
        from .resources.license_key_instances import LicenseKeyInstancesResource

        return LicenseKeyInstancesResource(self)

    @cached_property
    def customers(self) -> CustomersResource:
        from .resources.customers import CustomersResource

        return CustomersResource(self)

    @cached_property
    def refunds(self) -> RefundsResource:
        from .resources.refunds import RefundsResource

        return RefundsResource(self)

    @cached_property
    def disputes(self) -> DisputesResource:
        from .resources.disputes import DisputesResource

        return DisputesResource(self)

    @cached_property
    def payouts(self) -> PayoutsResource:
        from .resources.payouts import PayoutsResource

        return PayoutsResource(self)

    @cached_property
    def products(self) -> ProductsResource:
        from .resources.products import ProductsResource

        return ProductsResource(self)

    @cached_property
    def misc(self) -> MiscResource:
        from .resources.misc import MiscResource

        return MiscResource(self)

    @cached_property
    def discounts(self) -> DiscountsResource:
        from .resources.discounts import DiscountsResource

        return DiscountsResource(self)

    @cached_property
    def addons(self) -> AddonsResource:
        from .resources.addons import AddonsResource

        return AddonsResource(self)

    @cached_property
    def brands(self) -> BrandsResource:
        from .resources.brands import BrandsResource

        return BrandsResource(self)

    @cached_property
    def webhooks(self) -> WebhooksResource:
        from .resources.webhooks import WebhooksResource

        return WebhooksResource(self)

    @cached_property
    def usage_events(self) -> UsageEventsResource:
        from .resources.usage_events import UsageEventsResource

        return UsageEventsResource(self)

    @cached_property
    def meters(self) -> MetersResource:
        from .resources.meters import MetersResource

        return MetersResource(self)

    @cached_property
    def balances(self) -> BalancesResource:
        from .resources.balances import BalancesResource

        return BalancesResource(self)

    @cached_property
    def credit_entitlements(self) -> CreditEntitlementsResource:
        from .resources.credit_entitlements import CreditEntitlementsResource

        return CreditEntitlementsResource(self)

    @cached_property
    def with_raw_response(self) -> DodoPaymentsWithRawResponse:
        return DodoPaymentsWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DodoPaymentsWithStreamedResponse:
        return DodoPaymentsWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        bearer_token = self.bearer_token
        return {"Authorization": f"Bearer {bearer_token}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        bearer_token: str | None = None,
        webhook_key: str | None = None,
        environment: Literal["live_mode", "test_mode"] | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            bearer_token=bearer_token or self.bearer_token,
            webhook_key=webhook_key or self.webhook_key,
            base_url=base_url or self.base_url,
            environment=environment or self._environment,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncDodoPayments(AsyncAPIClient):
    # client options
    bearer_token: str
    webhook_key: str | None

    _environment: Literal["live_mode", "test_mode"] | NotGiven

    def __init__(
        self,
        *,
        bearer_token: str | None = None,
        webhook_key: str | None = None,
        environment: Literal["live_mode", "test_mode"] | NotGiven = not_given,
        base_url: str | httpx.URL | None | NotGiven = not_given,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncDodoPayments client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `bearer_token` from `DODO_PAYMENTS_API_KEY`
        - `webhook_key` from `DODO_PAYMENTS_WEBHOOK_KEY`
        """
        if bearer_token is None:
            bearer_token = os.environ.get("DODO_PAYMENTS_API_KEY")
        if bearer_token is None:
            raise DodoPaymentsError(
                "The bearer_token client option must be set either by passing bearer_token to the client or by setting the DODO_PAYMENTS_API_KEY environment variable"
            )
        self.bearer_token = bearer_token

        if webhook_key is None:
            webhook_key = os.environ.get("DODO_PAYMENTS_WEBHOOK_KEY")
        self.webhook_key = webhook_key

        self._environment = environment

        base_url_env = os.environ.get("DODO_PAYMENTS_BASE_URL")
        if is_given(base_url) and base_url is not None:
            # cast required because mypy doesn't understand the type narrowing
            base_url = cast("str | httpx.URL", base_url)  # pyright: ignore[reportUnnecessaryCast]
        elif is_given(environment):
            if base_url_env and base_url is not None:
                raise ValueError(
                    "Ambiguous URL; The `DODO_PAYMENTS_BASE_URL` env var and the `environment` argument are given. If you want to use the environment, you must pass base_url=None",
                )

            try:
                base_url = ENVIRONMENTS[environment]
            except KeyError as exc:
                raise ValueError(f"Unknown environment: {environment}") from exc
        elif base_url_env is not None:
            base_url = base_url_env
        else:
            self._environment = environment = "live_mode"

            try:
                base_url = ENVIRONMENTS[environment]
            except KeyError as exc:
                raise ValueError(f"Unknown environment: {environment}") from exc

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def checkout_sessions(self) -> AsyncCheckoutSessionsResource:
        from .resources.checkout_sessions import AsyncCheckoutSessionsResource

        return AsyncCheckoutSessionsResource(self)

    @cached_property
    def payments(self) -> AsyncPaymentsResource:
        from .resources.payments import AsyncPaymentsResource

        return AsyncPaymentsResource(self)

    @cached_property
    def subscriptions(self) -> AsyncSubscriptionsResource:
        from .resources.subscriptions import AsyncSubscriptionsResource

        return AsyncSubscriptionsResource(self)

    @cached_property
    def invoices(self) -> AsyncInvoicesResource:
        from .resources.invoices import AsyncInvoicesResource

        return AsyncInvoicesResource(self)

    @cached_property
    def licenses(self) -> AsyncLicensesResource:
        from .resources.licenses import AsyncLicensesResource

        return AsyncLicensesResource(self)

    @cached_property
    def license_keys(self) -> AsyncLicenseKeysResource:
        from .resources.license_keys import AsyncLicenseKeysResource

        return AsyncLicenseKeysResource(self)

    @cached_property
    def license_key_instances(self) -> AsyncLicenseKeyInstancesResource:
        from .resources.license_key_instances import AsyncLicenseKeyInstancesResource

        return AsyncLicenseKeyInstancesResource(self)

    @cached_property
    def customers(self) -> AsyncCustomersResource:
        from .resources.customers import AsyncCustomersResource

        return AsyncCustomersResource(self)

    @cached_property
    def refunds(self) -> AsyncRefundsResource:
        from .resources.refunds import AsyncRefundsResource

        return AsyncRefundsResource(self)

    @cached_property
    def disputes(self) -> AsyncDisputesResource:
        from .resources.disputes import AsyncDisputesResource

        return AsyncDisputesResource(self)

    @cached_property
    def payouts(self) -> AsyncPayoutsResource:
        from .resources.payouts import AsyncPayoutsResource

        return AsyncPayoutsResource(self)

    @cached_property
    def products(self) -> AsyncProductsResource:
        from .resources.products import AsyncProductsResource

        return AsyncProductsResource(self)

    @cached_property
    def misc(self) -> AsyncMiscResource:
        from .resources.misc import AsyncMiscResource

        return AsyncMiscResource(self)

    @cached_property
    def discounts(self) -> AsyncDiscountsResource:
        from .resources.discounts import AsyncDiscountsResource

        return AsyncDiscountsResource(self)

    @cached_property
    def addons(self) -> AsyncAddonsResource:
        from .resources.addons import AsyncAddonsResource

        return AsyncAddonsResource(self)

    @cached_property
    def brands(self) -> AsyncBrandsResource:
        from .resources.brands import AsyncBrandsResource

        return AsyncBrandsResource(self)

    @cached_property
    def webhooks(self) -> AsyncWebhooksResource:
        from .resources.webhooks import AsyncWebhooksResource

        return AsyncWebhooksResource(self)

    @cached_property
    def usage_events(self) -> AsyncUsageEventsResource:
        from .resources.usage_events import AsyncUsageEventsResource

        return AsyncUsageEventsResource(self)

    @cached_property
    def meters(self) -> AsyncMetersResource:
        from .resources.meters import AsyncMetersResource

        return AsyncMetersResource(self)

    @cached_property
    def balances(self) -> AsyncBalancesResource:
        from .resources.balances import AsyncBalancesResource

        return AsyncBalancesResource(self)

    @cached_property
    def credit_entitlements(self) -> AsyncCreditEntitlementsResource:
        from .resources.credit_entitlements import AsyncCreditEntitlementsResource

        return AsyncCreditEntitlementsResource(self)

    @cached_property
    def with_raw_response(self) -> AsyncDodoPaymentsWithRawResponse:
        return AsyncDodoPaymentsWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDodoPaymentsWithStreamedResponse:
        return AsyncDodoPaymentsWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        bearer_token = self.bearer_token
        return {"Authorization": f"Bearer {bearer_token}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        bearer_token: str | None = None,
        webhook_key: str | None = None,
        environment: Literal["live_mode", "test_mode"] | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            bearer_token=bearer_token or self.bearer_token,
            webhook_key=webhook_key or self.webhook_key,
            base_url=base_url or self.base_url,
            environment=environment or self._environment,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class DodoPaymentsWithRawResponse:
    _client: DodoPayments

    def __init__(self, client: DodoPayments) -> None:
        self._client = client

    @cached_property
    def checkout_sessions(self) -> checkout_sessions.CheckoutSessionsResourceWithRawResponse:
        from .resources.checkout_sessions import CheckoutSessionsResourceWithRawResponse

        return CheckoutSessionsResourceWithRawResponse(self._client.checkout_sessions)

    @cached_property
    def payments(self) -> payments.PaymentsResourceWithRawResponse:
        from .resources.payments import PaymentsResourceWithRawResponse

        return PaymentsResourceWithRawResponse(self._client.payments)

    @cached_property
    def subscriptions(self) -> subscriptions.SubscriptionsResourceWithRawResponse:
        from .resources.subscriptions import SubscriptionsResourceWithRawResponse

        return SubscriptionsResourceWithRawResponse(self._client.subscriptions)

    @cached_property
    def invoices(self) -> invoices.InvoicesResourceWithRawResponse:
        from .resources.invoices import InvoicesResourceWithRawResponse

        return InvoicesResourceWithRawResponse(self._client.invoices)

    @cached_property
    def licenses(self) -> licenses.LicensesResourceWithRawResponse:
        from .resources.licenses import LicensesResourceWithRawResponse

        return LicensesResourceWithRawResponse(self._client.licenses)

    @cached_property
    def license_keys(self) -> license_keys.LicenseKeysResourceWithRawResponse:
        from .resources.license_keys import LicenseKeysResourceWithRawResponse

        return LicenseKeysResourceWithRawResponse(self._client.license_keys)

    @cached_property
    def license_key_instances(self) -> license_key_instances.LicenseKeyInstancesResourceWithRawResponse:
        from .resources.license_key_instances import LicenseKeyInstancesResourceWithRawResponse

        return LicenseKeyInstancesResourceWithRawResponse(self._client.license_key_instances)

    @cached_property
    def customers(self) -> customers.CustomersResourceWithRawResponse:
        from .resources.customers import CustomersResourceWithRawResponse

        return CustomersResourceWithRawResponse(self._client.customers)

    @cached_property
    def refunds(self) -> refunds.RefundsResourceWithRawResponse:
        from .resources.refunds import RefundsResourceWithRawResponse

        return RefundsResourceWithRawResponse(self._client.refunds)

    @cached_property
    def disputes(self) -> disputes.DisputesResourceWithRawResponse:
        from .resources.disputes import DisputesResourceWithRawResponse

        return DisputesResourceWithRawResponse(self._client.disputes)

    @cached_property
    def payouts(self) -> payouts.PayoutsResourceWithRawResponse:
        from .resources.payouts import PayoutsResourceWithRawResponse

        return PayoutsResourceWithRawResponse(self._client.payouts)

    @cached_property
    def products(self) -> products.ProductsResourceWithRawResponse:
        from .resources.products import ProductsResourceWithRawResponse

        return ProductsResourceWithRawResponse(self._client.products)

    @cached_property
    def misc(self) -> misc.MiscResourceWithRawResponse:
        from .resources.misc import MiscResourceWithRawResponse

        return MiscResourceWithRawResponse(self._client.misc)

    @cached_property
    def discounts(self) -> discounts.DiscountsResourceWithRawResponse:
        from .resources.discounts import DiscountsResourceWithRawResponse

        return DiscountsResourceWithRawResponse(self._client.discounts)

    @cached_property
    def addons(self) -> addons.AddonsResourceWithRawResponse:
        from .resources.addons import AddonsResourceWithRawResponse

        return AddonsResourceWithRawResponse(self._client.addons)

    @cached_property
    def brands(self) -> brands.BrandsResourceWithRawResponse:
        from .resources.brands import BrandsResourceWithRawResponse

        return BrandsResourceWithRawResponse(self._client.brands)

    @cached_property
    def webhooks(self) -> webhooks.WebhooksResourceWithRawResponse:
        from .resources.webhooks import WebhooksResourceWithRawResponse

        return WebhooksResourceWithRawResponse(self._client.webhooks)

    @cached_property
    def usage_events(self) -> usage_events.UsageEventsResourceWithRawResponse:
        from .resources.usage_events import UsageEventsResourceWithRawResponse

        return UsageEventsResourceWithRawResponse(self._client.usage_events)

    @cached_property
    def meters(self) -> meters.MetersResourceWithRawResponse:
        from .resources.meters import MetersResourceWithRawResponse

        return MetersResourceWithRawResponse(self._client.meters)

    @cached_property
    def balances(self) -> balances.BalancesResourceWithRawResponse:
        from .resources.balances import BalancesResourceWithRawResponse

        return BalancesResourceWithRawResponse(self._client.balances)

    @cached_property
    def credit_entitlements(self) -> credit_entitlements.CreditEntitlementsResourceWithRawResponse:
        from .resources.credit_entitlements import CreditEntitlementsResourceWithRawResponse

        return CreditEntitlementsResourceWithRawResponse(self._client.credit_entitlements)


class AsyncDodoPaymentsWithRawResponse:
    _client: AsyncDodoPayments

    def __init__(self, client: AsyncDodoPayments) -> None:
        self._client = client

    @cached_property
    def checkout_sessions(self) -> checkout_sessions.AsyncCheckoutSessionsResourceWithRawResponse:
        from .resources.checkout_sessions import AsyncCheckoutSessionsResourceWithRawResponse

        return AsyncCheckoutSessionsResourceWithRawResponse(self._client.checkout_sessions)

    @cached_property
    def payments(self) -> payments.AsyncPaymentsResourceWithRawResponse:
        from .resources.payments import AsyncPaymentsResourceWithRawResponse

        return AsyncPaymentsResourceWithRawResponse(self._client.payments)

    @cached_property
    def subscriptions(self) -> subscriptions.AsyncSubscriptionsResourceWithRawResponse:
        from .resources.subscriptions import AsyncSubscriptionsResourceWithRawResponse

        return AsyncSubscriptionsResourceWithRawResponse(self._client.subscriptions)

    @cached_property
    def invoices(self) -> invoices.AsyncInvoicesResourceWithRawResponse:
        from .resources.invoices import AsyncInvoicesResourceWithRawResponse

        return AsyncInvoicesResourceWithRawResponse(self._client.invoices)

    @cached_property
    def licenses(self) -> licenses.AsyncLicensesResourceWithRawResponse:
        from .resources.licenses import AsyncLicensesResourceWithRawResponse

        return AsyncLicensesResourceWithRawResponse(self._client.licenses)

    @cached_property
    def license_keys(self) -> license_keys.AsyncLicenseKeysResourceWithRawResponse:
        from .resources.license_keys import AsyncLicenseKeysResourceWithRawResponse

        return AsyncLicenseKeysResourceWithRawResponse(self._client.license_keys)

    @cached_property
    def license_key_instances(self) -> license_key_instances.AsyncLicenseKeyInstancesResourceWithRawResponse:
        from .resources.license_key_instances import AsyncLicenseKeyInstancesResourceWithRawResponse

        return AsyncLicenseKeyInstancesResourceWithRawResponse(self._client.license_key_instances)

    @cached_property
    def customers(self) -> customers.AsyncCustomersResourceWithRawResponse:
        from .resources.customers import AsyncCustomersResourceWithRawResponse

        return AsyncCustomersResourceWithRawResponse(self._client.customers)

    @cached_property
    def refunds(self) -> refunds.AsyncRefundsResourceWithRawResponse:
        from .resources.refunds import AsyncRefundsResourceWithRawResponse

        return AsyncRefundsResourceWithRawResponse(self._client.refunds)

    @cached_property
    def disputes(self) -> disputes.AsyncDisputesResourceWithRawResponse:
        from .resources.disputes import AsyncDisputesResourceWithRawResponse

        return AsyncDisputesResourceWithRawResponse(self._client.disputes)

    @cached_property
    def payouts(self) -> payouts.AsyncPayoutsResourceWithRawResponse:
        from .resources.payouts import AsyncPayoutsResourceWithRawResponse

        return AsyncPayoutsResourceWithRawResponse(self._client.payouts)

    @cached_property
    def products(self) -> products.AsyncProductsResourceWithRawResponse:
        from .resources.products import AsyncProductsResourceWithRawResponse

        return AsyncProductsResourceWithRawResponse(self._client.products)

    @cached_property
    def misc(self) -> misc.AsyncMiscResourceWithRawResponse:
        from .resources.misc import AsyncMiscResourceWithRawResponse

        return AsyncMiscResourceWithRawResponse(self._client.misc)

    @cached_property
    def discounts(self) -> discounts.AsyncDiscountsResourceWithRawResponse:
        from .resources.discounts import AsyncDiscountsResourceWithRawResponse

        return AsyncDiscountsResourceWithRawResponse(self._client.discounts)

    @cached_property
    def addons(self) -> addons.AsyncAddonsResourceWithRawResponse:
        from .resources.addons import AsyncAddonsResourceWithRawResponse

        return AsyncAddonsResourceWithRawResponse(self._client.addons)

    @cached_property
    def brands(self) -> brands.AsyncBrandsResourceWithRawResponse:
        from .resources.brands import AsyncBrandsResourceWithRawResponse

        return AsyncBrandsResourceWithRawResponse(self._client.brands)

    @cached_property
    def webhooks(self) -> webhooks.AsyncWebhooksResourceWithRawResponse:
        from .resources.webhooks import AsyncWebhooksResourceWithRawResponse

        return AsyncWebhooksResourceWithRawResponse(self._client.webhooks)

    @cached_property
    def usage_events(self) -> usage_events.AsyncUsageEventsResourceWithRawResponse:
        from .resources.usage_events import AsyncUsageEventsResourceWithRawResponse

        return AsyncUsageEventsResourceWithRawResponse(self._client.usage_events)

    @cached_property
    def meters(self) -> meters.AsyncMetersResourceWithRawResponse:
        from .resources.meters import AsyncMetersResourceWithRawResponse

        return AsyncMetersResourceWithRawResponse(self._client.meters)

    @cached_property
    def balances(self) -> balances.AsyncBalancesResourceWithRawResponse:
        from .resources.balances import AsyncBalancesResourceWithRawResponse

        return AsyncBalancesResourceWithRawResponse(self._client.balances)

    @cached_property
    def credit_entitlements(self) -> credit_entitlements.AsyncCreditEntitlementsResourceWithRawResponse:
        from .resources.credit_entitlements import AsyncCreditEntitlementsResourceWithRawResponse

        return AsyncCreditEntitlementsResourceWithRawResponse(self._client.credit_entitlements)


class DodoPaymentsWithStreamedResponse:
    _client: DodoPayments

    def __init__(self, client: DodoPayments) -> None:
        self._client = client

    @cached_property
    def checkout_sessions(self) -> checkout_sessions.CheckoutSessionsResourceWithStreamingResponse:
        from .resources.checkout_sessions import CheckoutSessionsResourceWithStreamingResponse

        return CheckoutSessionsResourceWithStreamingResponse(self._client.checkout_sessions)

    @cached_property
    def payments(self) -> payments.PaymentsResourceWithStreamingResponse:
        from .resources.payments import PaymentsResourceWithStreamingResponse

        return PaymentsResourceWithStreamingResponse(self._client.payments)

    @cached_property
    def subscriptions(self) -> subscriptions.SubscriptionsResourceWithStreamingResponse:
        from .resources.subscriptions import SubscriptionsResourceWithStreamingResponse

        return SubscriptionsResourceWithStreamingResponse(self._client.subscriptions)

    @cached_property
    def invoices(self) -> invoices.InvoicesResourceWithStreamingResponse:
        from .resources.invoices import InvoicesResourceWithStreamingResponse

        return InvoicesResourceWithStreamingResponse(self._client.invoices)

    @cached_property
    def licenses(self) -> licenses.LicensesResourceWithStreamingResponse:
        from .resources.licenses import LicensesResourceWithStreamingResponse

        return LicensesResourceWithStreamingResponse(self._client.licenses)

    @cached_property
    def license_keys(self) -> license_keys.LicenseKeysResourceWithStreamingResponse:
        from .resources.license_keys import LicenseKeysResourceWithStreamingResponse

        return LicenseKeysResourceWithStreamingResponse(self._client.license_keys)

    @cached_property
    def license_key_instances(self) -> license_key_instances.LicenseKeyInstancesResourceWithStreamingResponse:
        from .resources.license_key_instances import LicenseKeyInstancesResourceWithStreamingResponse

        return LicenseKeyInstancesResourceWithStreamingResponse(self._client.license_key_instances)

    @cached_property
    def customers(self) -> customers.CustomersResourceWithStreamingResponse:
        from .resources.customers import CustomersResourceWithStreamingResponse

        return CustomersResourceWithStreamingResponse(self._client.customers)

    @cached_property
    def refunds(self) -> refunds.RefundsResourceWithStreamingResponse:
        from .resources.refunds import RefundsResourceWithStreamingResponse

        return RefundsResourceWithStreamingResponse(self._client.refunds)

    @cached_property
    def disputes(self) -> disputes.DisputesResourceWithStreamingResponse:
        from .resources.disputes import DisputesResourceWithStreamingResponse

        return DisputesResourceWithStreamingResponse(self._client.disputes)

    @cached_property
    def payouts(self) -> payouts.PayoutsResourceWithStreamingResponse:
        from .resources.payouts import PayoutsResourceWithStreamingResponse

        return PayoutsResourceWithStreamingResponse(self._client.payouts)

    @cached_property
    def products(self) -> products.ProductsResourceWithStreamingResponse:
        from .resources.products import ProductsResourceWithStreamingResponse

        return ProductsResourceWithStreamingResponse(self._client.products)

    @cached_property
    def misc(self) -> misc.MiscResourceWithStreamingResponse:
        from .resources.misc import MiscResourceWithStreamingResponse

        return MiscResourceWithStreamingResponse(self._client.misc)

    @cached_property
    def discounts(self) -> discounts.DiscountsResourceWithStreamingResponse:
        from .resources.discounts import DiscountsResourceWithStreamingResponse

        return DiscountsResourceWithStreamingResponse(self._client.discounts)

    @cached_property
    def addons(self) -> addons.AddonsResourceWithStreamingResponse:
        from .resources.addons import AddonsResourceWithStreamingResponse

        return AddonsResourceWithStreamingResponse(self._client.addons)

    @cached_property
    def brands(self) -> brands.BrandsResourceWithStreamingResponse:
        from .resources.brands import BrandsResourceWithStreamingResponse

        return BrandsResourceWithStreamingResponse(self._client.brands)

    @cached_property
    def webhooks(self) -> webhooks.WebhooksResourceWithStreamingResponse:
        from .resources.webhooks import WebhooksResourceWithStreamingResponse

        return WebhooksResourceWithStreamingResponse(self._client.webhooks)

    @cached_property
    def usage_events(self) -> usage_events.UsageEventsResourceWithStreamingResponse:
        from .resources.usage_events import UsageEventsResourceWithStreamingResponse

        return UsageEventsResourceWithStreamingResponse(self._client.usage_events)

    @cached_property
    def meters(self) -> meters.MetersResourceWithStreamingResponse:
        from .resources.meters import MetersResourceWithStreamingResponse

        return MetersResourceWithStreamingResponse(self._client.meters)

    @cached_property
    def balances(self) -> balances.BalancesResourceWithStreamingResponse:
        from .resources.balances import BalancesResourceWithStreamingResponse

        return BalancesResourceWithStreamingResponse(self._client.balances)

    @cached_property
    def credit_entitlements(self) -> credit_entitlements.CreditEntitlementsResourceWithStreamingResponse:
        from .resources.credit_entitlements import CreditEntitlementsResourceWithStreamingResponse

        return CreditEntitlementsResourceWithStreamingResponse(self._client.credit_entitlements)


class AsyncDodoPaymentsWithStreamedResponse:
    _client: AsyncDodoPayments

    def __init__(self, client: AsyncDodoPayments) -> None:
        self._client = client

    @cached_property
    def checkout_sessions(self) -> checkout_sessions.AsyncCheckoutSessionsResourceWithStreamingResponse:
        from .resources.checkout_sessions import AsyncCheckoutSessionsResourceWithStreamingResponse

        return AsyncCheckoutSessionsResourceWithStreamingResponse(self._client.checkout_sessions)

    @cached_property
    def payments(self) -> payments.AsyncPaymentsResourceWithStreamingResponse:
        from .resources.payments import AsyncPaymentsResourceWithStreamingResponse

        return AsyncPaymentsResourceWithStreamingResponse(self._client.payments)

    @cached_property
    def subscriptions(self) -> subscriptions.AsyncSubscriptionsResourceWithStreamingResponse:
        from .resources.subscriptions import AsyncSubscriptionsResourceWithStreamingResponse

        return AsyncSubscriptionsResourceWithStreamingResponse(self._client.subscriptions)

    @cached_property
    def invoices(self) -> invoices.AsyncInvoicesResourceWithStreamingResponse:
        from .resources.invoices import AsyncInvoicesResourceWithStreamingResponse

        return AsyncInvoicesResourceWithStreamingResponse(self._client.invoices)

    @cached_property
    def licenses(self) -> licenses.AsyncLicensesResourceWithStreamingResponse:
        from .resources.licenses import AsyncLicensesResourceWithStreamingResponse

        return AsyncLicensesResourceWithStreamingResponse(self._client.licenses)

    @cached_property
    def license_keys(self) -> license_keys.AsyncLicenseKeysResourceWithStreamingResponse:
        from .resources.license_keys import AsyncLicenseKeysResourceWithStreamingResponse

        return AsyncLicenseKeysResourceWithStreamingResponse(self._client.license_keys)

    @cached_property
    def license_key_instances(self) -> license_key_instances.AsyncLicenseKeyInstancesResourceWithStreamingResponse:
        from .resources.license_key_instances import AsyncLicenseKeyInstancesResourceWithStreamingResponse

        return AsyncLicenseKeyInstancesResourceWithStreamingResponse(self._client.license_key_instances)

    @cached_property
    def customers(self) -> customers.AsyncCustomersResourceWithStreamingResponse:
        from .resources.customers import AsyncCustomersResourceWithStreamingResponse

        return AsyncCustomersResourceWithStreamingResponse(self._client.customers)

    @cached_property
    def refunds(self) -> refunds.AsyncRefundsResourceWithStreamingResponse:
        from .resources.refunds import AsyncRefundsResourceWithStreamingResponse

        return AsyncRefundsResourceWithStreamingResponse(self._client.refunds)

    @cached_property
    def disputes(self) -> disputes.AsyncDisputesResourceWithStreamingResponse:
        from .resources.disputes import AsyncDisputesResourceWithStreamingResponse

        return AsyncDisputesResourceWithStreamingResponse(self._client.disputes)

    @cached_property
    def payouts(self) -> payouts.AsyncPayoutsResourceWithStreamingResponse:
        from .resources.payouts import AsyncPayoutsResourceWithStreamingResponse

        return AsyncPayoutsResourceWithStreamingResponse(self._client.payouts)

    @cached_property
    def products(self) -> products.AsyncProductsResourceWithStreamingResponse:
        from .resources.products import AsyncProductsResourceWithStreamingResponse

        return AsyncProductsResourceWithStreamingResponse(self._client.products)

    @cached_property
    def misc(self) -> misc.AsyncMiscResourceWithStreamingResponse:
        from .resources.misc import AsyncMiscResourceWithStreamingResponse

        return AsyncMiscResourceWithStreamingResponse(self._client.misc)

    @cached_property
    def discounts(self) -> discounts.AsyncDiscountsResourceWithStreamingResponse:
        from .resources.discounts import AsyncDiscountsResourceWithStreamingResponse

        return AsyncDiscountsResourceWithStreamingResponse(self._client.discounts)

    @cached_property
    def addons(self) -> addons.AsyncAddonsResourceWithStreamingResponse:
        from .resources.addons import AsyncAddonsResourceWithStreamingResponse

        return AsyncAddonsResourceWithStreamingResponse(self._client.addons)

    @cached_property
    def brands(self) -> brands.AsyncBrandsResourceWithStreamingResponse:
        from .resources.brands import AsyncBrandsResourceWithStreamingResponse

        return AsyncBrandsResourceWithStreamingResponse(self._client.brands)

    @cached_property
    def webhooks(self) -> webhooks.AsyncWebhooksResourceWithStreamingResponse:
        from .resources.webhooks import AsyncWebhooksResourceWithStreamingResponse

        return AsyncWebhooksResourceWithStreamingResponse(self._client.webhooks)

    @cached_property
    def usage_events(self) -> usage_events.AsyncUsageEventsResourceWithStreamingResponse:
        from .resources.usage_events import AsyncUsageEventsResourceWithStreamingResponse

        return AsyncUsageEventsResourceWithStreamingResponse(self._client.usage_events)

    @cached_property
    def meters(self) -> meters.AsyncMetersResourceWithStreamingResponse:
        from .resources.meters import AsyncMetersResourceWithStreamingResponse

        return AsyncMetersResourceWithStreamingResponse(self._client.meters)

    @cached_property
    def balances(self) -> balances.AsyncBalancesResourceWithStreamingResponse:
        from .resources.balances import AsyncBalancesResourceWithStreamingResponse

        return AsyncBalancesResourceWithStreamingResponse(self._client.balances)

    @cached_property
    def credit_entitlements(self) -> credit_entitlements.AsyncCreditEntitlementsResourceWithStreamingResponse:
        from .resources.credit_entitlements import AsyncCreditEntitlementsResourceWithStreamingResponse

        return AsyncCreditEntitlementsResourceWithStreamingResponse(self._client.credit_entitlements)


Client = DodoPayments

AsyncClient = AsyncDodoPayments



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .header_update_params import HeaderUpdateParams as HeaderUpdateParams
from .header_retrieve_response import HeaderRetrieveResponse as HeaderRetrieveResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["SubscriptionRetrieveCreditUsageResponse", "Item"]


class Item(BaseModel):
    """Per-entitlement credit usage status for a subscription"""

    balance: str
    """Customer's current credit balance for this entitlement (customer-wide)"""

    credit_entitlement_id: str

    credit_entitlement_name: str

    limit_reached: bool
    """
    True if overage has reached or exceeded the limit. When true, further deductions
    that would increase overage will fail.
    """

    overage: str
    """Current overage amount accrued (customer-wide)"""

    overage_enabled: bool
    """Whether overage is enabled for this entitlement on this subscription"""

    unit: str
    """Unit label for the credit entitlement (e.g. "API Calls", "Tokens")"""

    overage_limit: Optional[str] = None
    """
    Maximum allowed overage before deductions are blocked. None means unlimited
    overage (when overage_enabled is true).
    """

    remaining_headroom: Optional[str] = None
    """
    How much more overage can accumulate before being blocked. None if overage is
    not enabled or there is no limit (unlimited). A value of 0 means the next
    deduction that increases overage will be blocked.
    """


class SubscriptionRetrieveCreditUsageResponse(BaseModel):
    """Credit usage status for all entitlements linked to a subscription"""

    items: List[Item]

    subscription_id: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Union, Optional
from datetime import datetime

from .._models import BaseModel

__all__ = ["Event"]


class Event(BaseModel):
    business_id: str

    customer_id: str

    event_id: str

    event_name: str

    timestamp: datetime

    metadata: Optional[Dict[str, Union[str, float, bool]]] = None
    """Arbitrary key-value metadata.

    Values can be string, integer, number, or boolean.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from datetime import datetime
from typing_extensions import Literal

import httpx

from ..types import license_key_list_params, license_key_update_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultPageNumberPagination, AsyncDefaultPageNumberPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.license_key import LicenseKey

__all__ = ["LicenseKeysResource", "AsyncLicenseKeysResource"]


class LicenseKeysResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> LicenseKeysResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#accessing-raw-response-data-eg-headers
        """
        return LicenseKeysResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LicenseKeysResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#with_streaming_response
        """
        return LicenseKeysResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LicenseKey:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/license_keys/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LicenseKey,
        )

    def update(
        self,
        id: str,
        *,
        activations_limit: Optional[int] | Omit = omit,
        disabled: Optional[bool] | Omit = omit,
        expires_at: Union[str, datetime, None] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LicenseKey:
        """Args:
          activations_limit: The updated activation limit for the license key.

        Use `null` to remove the
              limit, or omit this field to leave it unchanged.

          disabled: Indicates whether the license key should be disabled. A value of `true` disables
              the key, while `false` enables it. Omit this field to leave it unchanged.

          expires_at: The updated expiration timestamp for the license key in UTC. Use `null` to
              remove the expiration date, or omit this field to leave it unchanged.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/license_keys/{id}",
            body=maybe_transform(
                {
                    "activations_limit": activations_limit,
                    "disabled": disabled,
                    "expires_at": expires_at,
                },
                license_key_update_params.LicenseKeyUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LicenseKey,
        )

    def list(
        self,
        *,
        created_at_gte: Union[str, datetime] | Omit = omit,
        created_at_lte: Union[str, datetime] | Omit = omit,
        customer_id: str | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        product_id: str | Omit = omit,
        status: Literal["active", "expired", "disabled"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultPageNumberPagination[LicenseKey]:
        """
        Args:
          created_at_gte: Filter license keys created on or after this timestamp

          created_at_lte: Filter license keys created on or before this timestamp

          customer_id: Filter by customer ID

          page_number: Page number default is 0

          page_size: Page size default is 10 max is 100

          product_id: Filter by product ID

          status: Filter by license key status

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/license_keys",
            page=SyncDefaultPageNumberPagination[LicenseKey],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "created_at_gte": created_at_gte,
                        "created_at_lte": created_at_lte,
                        "customer_id": customer_id,
                        "page_number": page_number,
                        "page_size": page_size,
                        "product_id": product_id,
                        "status": status,
                    },
                    license_key_list_params.LicenseKeyListParams,
                ),
            ),
            model=LicenseKey,
        )


class AsyncLicenseKeysResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncLicenseKeysResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#accessing-raw-response-data-eg-headers
        """
        return AsyncLicenseKeysResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLicenseKeysResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#with_streaming_response
        """
        return AsyncLicenseKeysResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LicenseKey:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/license_keys/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LicenseKey,
        )

    async def update(
        self,
        id: str,
        *,
        activations_limit: Optional[int] | Omit = omit,
        disabled: Optional[bool] | Omit = omit,
        expires_at: Union[str, datetime, None] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LicenseKey:
        """Args:
          activations_limit: The updated activation limit for the license key.

        Use `null` to remove the
              limit, or omit this field to leave it unchanged.

          disabled: Indicates whether the license key should be disabled. A value of `true` disables
              the key, while `false` enables it. Omit this field to leave it unchanged.

          expires_at: The updated expiration timestamp for the license key in UTC. Use `null` to
              remove the expiration date, or omit this field to leave it unchanged.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/license_keys/{id}",
            body=await async_maybe_transform(
                {
                    "activations_limit": activations_limit,
                    "disabled": disabled,
                    "expires_at": expires_at,
                },
                license_key_update_params.LicenseKeyUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LicenseKey,
        )

    def list(
        self,
        *,
        created_at_gte: Union[str, datetime] | Omit = omit,
        created_at_lte: Union[str, datetime] | Omit = omit,
        customer_id: str | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        product_id: str | Omit = omit,
        status: Literal["active", "expired", "disabled"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[LicenseKey, AsyncDefaultPageNumberPagination[LicenseKey]]:
        """
        Args:
          created_at_gte: Filter license keys created on or after this timestamp

          created_at_lte: Filter license keys created on or before this timestamp

          customer_id: Filter by customer ID

          page_number: Page number default is 0

          page_size: Page size default is 10 max is 100

          product_id: Filter by product ID

          status: Filter by license key status

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/license_keys",
            page=AsyncDefaultPageNumberPagination[LicenseKey],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "created_at_gte": created_at_gte,
                        "created_at_lte": created_at_lte,
                        "customer_id": customer_id,
                        "page_number": page_number,
                        "page_size": page_size,
                        "product_id": product_id,
                        "status": status,
                    },
                    license_key_list_params.LicenseKeyListParams,
                ),
            ),
            model=LicenseKey,
        )


class LicenseKeysResourceWithRawResponse:
    def __init__(self, license_keys: LicenseKeysResource) -> None:
        self._license_keys = license_keys

        self.retrieve = to_raw_response_wrapper(
            license_keys.retrieve,
        )
        self.update = to_raw_response_wrapper(
            license_keys.update,
        )
        self.list = to_raw_response_wrapper(
            license_keys.list,
        )


class AsyncLicenseKeysResourceWithRawResponse:
    def __init__(self, license_keys: AsyncLicenseKeysResource) -> None:
        self._license_keys = license_keys

        self.retrieve = async_to_raw_response_wrapper(
            license_keys.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            license_keys.update,
        )
        self.list = async_to_raw_response_wrapper(
            license_keys.list,
        )


class LicenseKeysResourceWithStreamingResponse:
    def __init__(self, license_keys: LicenseKeysResource) -> None:
        self._license_keys = license_keys

        self.retrieve = to_streamed_response_wrapper(
            license_keys.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            license_keys.update,
        )
        self.list = to_streamed_response_wrapper(
            license_keys.list,
        )


class AsyncLicenseKeysResourceWithStreamingResponse:
    def __init__(self, license_keys: AsyncLicenseKeysResource) -> None:
        self._license_keys = license_keys

        self.retrieve = async_to_streamed_response_wrapper(
            license_keys.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            license_keys.update,
        )
        self.list = async_to_streamed_response_wrapper(
            license_keys.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["BalanceListLedgerParams"]


class BalanceListLedgerParams(TypedDict, total=False):
    credit_entitlement_id: Required[str]

    end_date: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter by end date"""

    page_number: int
    """Page number default is 0"""

    page_size: int
    """Page size default is 10 max is 100"""

    start_date: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter by start date"""

    transaction_type: str
    """
    Filter by transaction type (snake_case: credit_added, credit_deducted,
    credit_expired, etc.)
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["CreditLedgerEntry"]


class CreditLedgerEntry(BaseModel):
    """Response for a ledger entry"""

    id: str

    amount: str

    balance_after: str

    balance_before: str

    business_id: str

    created_at: datetime

    credit_entitlement_id: str

    customer_id: str

    is_credit: bool

    overage_after: str

    overage_before: str

    transaction_type: Literal[
        "credit_added",
        "credit_deducted",
        "credit_expired",
        "credit_rolled_over",
        "rollover_forfeited",
        "overage_charged",
        "auto_top_up",
        "manual_adjustment",
        "refund",
    ]

    description: Optional[str] = None

    grant_id: Optional[str] = None

    reference_id: Optional[str] = None

    reference_type: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ..types import brand_create_params, brand_update_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..types.brand import Brand
from .._base_client import make_request_options
from ..types.brand_list_response import BrandListResponse
from ..types.brand_update_images_response import BrandUpdateImagesResponse

__all__ = ["BrandsResource", "AsyncBrandsResource"]


class BrandsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> BrandsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#accessing-raw-response-data-eg-headers
        """
        return BrandsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BrandsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#with_streaming_response
        """
        return BrandsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        description: Optional[str] | Omit = omit,
        name: Optional[str] | Omit = omit,
        statement_descriptor: Optional[str] | Omit = omit,
        support_email: Optional[str] | Omit = omit,
        url: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Brand:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/brands",
            body=maybe_transform(
                {
                    "description": description,
                    "name": name,
                    "statement_descriptor": statement_descriptor,
                    "support_email": support_email,
                    "url": url,
                },
                brand_create_params.BrandCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Brand,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Brand:
        """
        Thin handler just calls `get_brand` and wraps in `Json(...)`

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/brands/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Brand,
        )

    def update(
        self,
        id: str,
        *,
        description: Optional[str] | Omit = omit,
        image_id: Optional[str] | Omit = omit,
        name: Optional[str] | Omit = omit,
        statement_descriptor: Optional[str] | Omit = omit,
        support_email: Optional[str] | Omit = omit,
        url: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Brand:
        """
        Args:
          image_id: The UUID you got back from the presigned‐upload call

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/brands/{id}",
            body=maybe_transform(
                {
                    "description": description,
                    "image_id": image_id,
                    "name": name,
                    "statement_descriptor": statement_descriptor,
                    "support_email": support_email,
                    "url": url,
                },
                brand_update_params.BrandUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Brand,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BrandListResponse:
        return self._get(
            "/brands",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BrandListResponse,
        )

    def update_images(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BrandUpdateImagesResponse:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/brands/{id}/images",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BrandUpdateImagesResponse,
        )


class AsyncBrandsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncBrandsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#accessing-raw-response-data-eg-headers
        """
        return AsyncBrandsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBrandsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#with_streaming_response
        """
        return AsyncBrandsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        description: Optional[str] | Omit = omit,
        name: Optional[str] | Omit = omit,
        statement_descriptor: Optional[str] | Omit = omit,
        support_email: Optional[str] | Omit = omit,
        url: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Brand:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/brands",
            body=await async_maybe_transform(
                {
                    "description": description,
                    "name": name,
                    "statement_descriptor": statement_descriptor,
                    "support_email": support_email,
                    "url": url,
                },
                brand_create_params.BrandCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Brand,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Brand:
        """
        Thin handler just calls `get_brand` and wraps in `Json(...)`

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/brands/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Brand,
        )

    async def update(
        self,
        id: str,
        *,
        description: Optional[str] | Omit = omit,
        image_id: Optional[str] | Omit = omit,
        name: Optional[str] | Omit = omit,
        statement_descriptor: Optional[str] | Omit = omit,
        support_email: Optional[str] | Omit = omit,
        url: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Brand:
        """
        Args:
          image_id: The UUID you got back from the presigned‐upload call

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/brands/{id}",
            body=await async_maybe_transform(
                {
                    "description": description,
                    "image_id": image_id,
                    "name": name,
                    "statement_descriptor": statement_descriptor,
                    "support_email": support_email,
                    "url": url,
                },
                brand_update_params.BrandUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Brand,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BrandListResponse:
        return await self._get(
            "/brands",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BrandListResponse,
        )

    async def update_images(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BrandUpdateImagesResponse:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/brands/{id}/images",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BrandUpdateImagesResponse,
        )


class BrandsResourceWithRawResponse:
    def __init__(self, brands: BrandsResource) -> None:
        self._brands = brands

        self.create = to_raw_response_wrapper(
            brands.create,
        )
        self.retrieve = to_raw_response_wrapper(
            brands.retrieve,
        )
        self.update = to_raw_response_wrapper(
            brands.update,
        )
        self.list = to_raw_response_wrapper(
            brands.list,
        )
        self.update_images = to_raw_response_wrapper(
            brands.update_images,
        )


class AsyncBrandsResourceWithRawResponse:
    def __init__(self, brands: AsyncBrandsResource) -> None:
        self._brands = brands

        self.create = async_to_raw_response_wrapper(
            brands.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            brands.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            brands.update,
        )
        self.list = async_to_raw_response_wrapper(
            brands.list,
        )
        self.update_images = async_to_raw_response_wrapper(
            brands.update_images,
        )


class BrandsResourceWithStreamingResponse:
    def __init__(self, brands: BrandsResource) -> None:
        self._brands = brands

        self.create = to_streamed_response_wrapper(
            brands.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            brands.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            brands.update,
        )
        self.list = to_streamed_response_wrapper(
            brands.list,
        )
        self.update_images = to_streamed_response_wrapper(
            brands.update_images,
        )


class AsyncBrandsResourceWithStreamingResponse:
    def __init__(self, brands: AsyncBrandsResource) -> None:
        self._brands = brands

        self.create = async_to_streamed_response_wrapper(
            brands.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            brands.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            brands.update,
        )
        self.list = async_to_streamed_response_wrapper(
            brands.list,
        )
        self.update_images = async_to_streamed_response_wrapper(
            brands.update_images,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .ledger_entry_type import LedgerEntryType as LedgerEntryType
from .balance_list_params import BalanceListParams as BalanceListParams
from .credit_ledger_entry import CreditLedgerEntry as CreditLedgerEntry
from .customer_credit_balance import CustomerCreditBalance as CustomerCreditBalance
from .balance_list_grants_params import BalanceListGrantsParams as BalanceListGrantsParams
from .balance_list_ledger_params import BalanceListLedgerParams as BalanceListLedgerParams
from .balance_list_grants_response import BalanceListGrantsResponse as BalanceListGrantsResponse
from .balance_create_ledger_entry_params import BalanceCreateLedgerEntryParams as BalanceCreateLedgerEntryParams
from .balance_create_ledger_entry_response import BalanceCreateLedgerEntryResponse as BalanceCreateLedgerEntryResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from dodopayments import DodoPayments, AsyncDodoPayments
from dodopayments.types import (
    Payment,
    PaymentListResponse,
    PaymentCreateResponse,
    PaymentRetrieveLineItemsResponse,
)
from dodopayments._utils import parse_datetime
from dodopayments.pagination import SyncDefaultPageNumberPagination, AsyncDefaultPageNumberPagination

# pyright: reportDeprecated=false

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestPayments:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: DodoPayments) -> None:
        with pytest.warns(DeprecationWarning):
            payment = client.payments.create(
                billing={"country": "AF"},
                customer={"customer_id": "customer_id"},
                product_cart=[
                    {
                        "product_id": "product_id",
                        "quantity": 0,
                    }
                ],
            )

        assert_matches_type(PaymentCreateResponse, payment, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: DodoPayments) -> None:
        with pytest.warns(DeprecationWarning):
            payment = client.payments.create(
                billing={
                    "country": "AF",
                    "city": "city",
                    "state": "state",
                    "street": "street",
                    "zipcode": "zipcode",
                },
                customer={"customer_id": "customer_id"},
                product_cart=[
                    {
                        "product_id": "product_id",
                        "quantity": 0,
                        "amount": 0,
                    }
                ],
                allowed_payment_method_types=["ach"],
                billing_currency="AED",
                discount_code="discount_code",
                force_3ds=True,
                metadata={"foo": "string"},
                payment_link=True,
                payment_method_id="payment_method_id",
                redirect_immediately=True,
                return_url="return_url",
                short_link=True,
                show_saved_payment_methods=True,
                tax_id="tax_id",
            )

        assert_matches_type(PaymentCreateResponse, payment, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: DodoPayments) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.payments.with_raw_response.create(
                billing={"country": "AF"},
                customer={"customer_id": "customer_id"},
                product_cart=[
                    {
                        "product_id": "product_id",
                        "quantity": 0,
                    }
                ],
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        payment = response.parse()
        assert_matches_type(PaymentCreateResponse, payment, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: DodoPayments) -> None:
        with pytest.warns(DeprecationWarning):
            with client.payments.with_streaming_response.create(
                billing={"country": "AF"},
                customer={"customer_id": "customer_id"},
                product_cart=[
                    {
                        "product_id": "product_id",
                        "quantity": 0,
                    }
                ],
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                payment = response.parse()
                assert_matches_type(PaymentCreateResponse, payment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: DodoPayments) -> None:
        payment = client.payments.retrieve(
            "payment_id",
        )
        assert_matches_type(Payment, payment, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: DodoPayments) -> None:
        response = client.payments.with_raw_response.retrieve(
            "payment_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        payment = response.parse()
        assert_matches_type(Payment, payment, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: DodoPayments) -> None:
        with client.payments.with_streaming_response.retrieve(
            "payment_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            payment = response.parse()
            assert_matches_type(Payment, payment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: DodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `payment_id` but received ''"):
            client.payments.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_list(self, client: DodoPayments) -> None:
        payment = client.payments.list()
        assert_matches_type(SyncDefaultPageNumberPagination[PaymentListResponse], payment, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: DodoPayments) -> None:
        payment = client.payments.list(
            brand_id="brand_id",
            created_at_gte=parse_datetime("2019-12-27T18:11:19.117Z"),
            created_at_lte=parse_datetime("2019-12-27T18:11:19.117Z"),
            customer_id="customer_id",
            page_number=0,
            page_size=0,
            product_id="product_id",
            status="succeeded",
            subscription_id="subscription_id",
        )
        assert_matches_type(SyncDefaultPageNumberPagination[PaymentListResponse], payment, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: DodoPayments) -> None:
        response = client.payments.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        payment = response.parse()
        assert_matches_type(SyncDefaultPageNumberPagination[PaymentListResponse], payment, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: DodoPayments) -> None:
        with client.payments.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            payment = response.parse()
            assert_matches_type(SyncDefaultPageNumberPagination[PaymentListResponse], payment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve_line_items(self, client: DodoPayments) -> None:
        payment = client.payments.retrieve_line_items(
            "payment_id",
        )
        assert_matches_type(PaymentRetrieveLineItemsResponse, payment, path=["response"])

    @parametrize
    def test_raw_response_retrieve_line_items(self, client: DodoPayments) -> None:
        response = client.payments.with_raw_response.retrieve_line_items(
            "payment_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        payment = response.parse()
        assert_matches_type(PaymentRetrieveLineItemsResponse, payment, path=["response"])

    @parametrize
    def test_streaming_response_retrieve_line_items(self, client: DodoPayments) -> None:
        with client.payments.with_streaming_response.retrieve_line_items(
            "payment_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            payment = response.parse()
            assert_matches_type(PaymentRetrieveLineItemsResponse, payment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve_line_items(self, client: DodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `payment_id` but received ''"):
            client.payments.with_raw_response.retrieve_line_items(
                "",
            )


class TestAsyncPayments:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncDodoPayments) -> None:
        with pytest.warns(DeprecationWarning):
            payment = await async_client.payments.create(
                billing={"country": "AF"},
                customer={"customer_id": "customer_id"},
                product_cart=[
                    {
                        "product_id": "product_id",
                        "quantity": 0,
                    }
                ],
            )

        assert_matches_type(PaymentCreateResponse, payment, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncDodoPayments) -> None:
        with pytest.warns(DeprecationWarning):
            payment = await async_client.payments.create(
                billing={
                    "country": "AF",
                    "city": "city",
                    "state": "state",
                    "street": "street",
                    "zipcode": "zipcode",
                },
                customer={"customer_id": "customer_id"},
                product_cart=[
                    {
                        "product_id": "product_id",
                        "quantity": 0,
                        "amount": 0,
                    }
                ],
                allowed_payment_method_types=["ach"],
                billing_currency="AED",
                discount_code="discount_code",
                force_3ds=True,
                metadata={"foo": "string"},
                payment_link=True,
                payment_method_id="payment_method_id",
                redirect_immediately=True,
                return_url="return_url",
                short_link=True,
                show_saved_payment_methods=True,
                tax_id="tax_id",
            )

        assert_matches_type(PaymentCreateResponse, payment, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncDodoPayments) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.payments.with_raw_response.create(
                billing={"country": "AF"},
                customer={"customer_id": "customer_id"},
                product_cart=[
                    {
                        "product_id": "product_id",
                        "quantity": 0,
                    }
                ],
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        payment = await response.parse()
        assert_matches_type(PaymentCreateResponse, payment, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncDodoPayments) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.payments.with_streaming_response.create(
                billing={"country": "AF"},
                customer={"customer_id": "customer_id"},
                product_cart=[
                    {
                        "product_id": "product_id",
                        "quantity": 0,
                    }
                ],
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                payment = await response.parse()
                assert_matches_type(PaymentCreateResponse, payment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncDodoPayments) -> None:
        payment = await async_client.payments.retrieve(
            "payment_id",
        )
        assert_matches_type(Payment, payment, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncDodoPayments) -> None:
        response = await async_client.payments.with_raw_response.retrieve(
            "payment_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        payment = await response.parse()
        assert_matches_type(Payment, payment, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncDodoPayments) -> None:
        async with async_client.payments.with_streaming_response.retrieve(
            "payment_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            payment = await response.parse()
            assert_matches_type(Payment, payment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncDodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `payment_id` but received ''"):
            await async_client.payments.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncDodoPayments) -> None:
        payment = await async_client.payments.list()
        assert_matches_type(AsyncDefaultPageNumberPagination[PaymentListResponse], payment, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncDodoPayments) -> None:
        payment = await async_client.payments.list(
            brand_id="brand_id",
            created_at_gte=parse_datetime("2019-12-27T18:11:19.117Z"),
            created_at_lte=parse_datetime("2019-12-27T18:11:19.117Z"),
            customer_id="customer_id",
            page_number=0,
            page_size=0,
            product_id="product_id",
            status="succeeded",
            subscription_id="subscription_id",
        )
        assert_matches_type(AsyncDefaultPageNumberPagination[PaymentListResponse], payment, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncDodoPayments) -> None:
        response = await async_client.payments.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        payment = await response.parse()
        assert_matches_type(AsyncDefaultPageNumberPagination[PaymentListResponse], payment, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncDodoPayments) -> None:
        async with async_client.payments.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            payment = await response.parse()
            assert_matches_type(AsyncDefaultPageNumberPagination[PaymentListResponse], payment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve_line_items(self, async_client: AsyncDodoPayments) -> None:
        payment = await async_client.payments.retrieve_line_items(
            "payment_id",
        )
        assert_matches_type(PaymentRetrieveLineItemsResponse, payment, path=["response"])

    @parametrize
    async def test_raw_response_retrieve_line_items(self, async_client: AsyncDodoPayments) -> None:
        response = await async_client.payments.with_raw_response.retrieve_line_items(
            "payment_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        payment = await response.parse()
        assert_matches_type(PaymentRetrieveLineItemsResponse, payment, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve_line_items(self, async_client: AsyncDodoPayments) -> None:
        async with async_client.payments.with_streaming_response.retrieve_line_items(
            "payment_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            payment = await response.parse()
            assert_matches_type(PaymentRetrieveLineItemsResponse, payment, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve_line_items(self, async_client: AsyncDodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `payment_id` but received ''"):
            await async_client.payments.with_raw_response.retrieve_line_items(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Iterable, Optional
from typing_extensions import Required, TypedDict

from .currency import Currency
from .custom_field_param import CustomFieldParam
from .payment_method_types import PaymentMethodTypes
from .customer_request_param import CustomerRequestParam
from .product_item_req_param import ProductItemReqParam
from .subscription_data_param import SubscriptionDataParam
from .checkout_session_flags_param import CheckoutSessionFlagsParam
from .checkout_session_customization_param import CheckoutSessionCustomizationParam
from .checkout_session_billing_address_param import CheckoutSessionBillingAddressParam

__all__ = ["CheckoutSessionCreateParams"]


class CheckoutSessionCreateParams(TypedDict, total=False):
    product_cart: Required[Iterable[ProductItemReqParam]]

    allowed_payment_method_types: Optional[List[PaymentMethodTypes]]
    """
    Customers will never see payment methods that are not in this list. However,
    adding a method here does not guarantee customers will see it. Availability
    still depends on other factors (e.g., customer location, merchant settings).

    Disclaimar: Always provide 'credit' and 'debit' as a fallback. If all payment
    methods are unavailable, checkout session will fail.
    """

    billing_address: Optional[CheckoutSessionBillingAddressParam]
    """Billing address information for the session"""

    billing_currency: Optional[Currency]
    """This field is ingored if adaptive pricing is disabled"""

    confirm: bool
    """If confirm is true, all the details will be finalized.

    If required data is missing, an API error is thrown.
    """

    custom_fields: Optional[Iterable[CustomFieldParam]]
    """Custom fields to collect from customer during checkout (max 5 fields)"""

    customer: Optional[CustomerRequestParam]
    """Customer details for the session"""

    customization: CheckoutSessionCustomizationParam
    """Customization for the checkout session page"""

    discount_code: Optional[str]

    feature_flags: CheckoutSessionFlagsParam

    force_3ds: Optional[bool]
    """Override merchant default 3DS behaviour for this session"""

    metadata: Optional[Dict[str, str]]
    """Additional metadata associated with the payment.

    Defaults to empty if not provided.
    """

    minimal_address: bool
    """
    If true, only zipcode is required when confirm is true; other address fields
    remain optional
    """

    payment_method_id: Optional[str]
    """
    Optional payment method ID to use for this checkout session. Only allowed when
    `confirm` is true. If provided, existing customer id must also be provided.
    """

    product_collection_id: Optional[str]
    """Product collection ID for collection-based checkout flow"""

    return_url: Optional[str]
    """The url to redirect after payment failure or success."""

    short_link: bool
    """If true, returns a shortened checkout URL. Defaults to false if not specified."""

    show_saved_payment_methods: bool
    """Display saved payment methods of a returning customer False by default"""

    subscription_data: Optional[SubscriptionDataParam]

    tax_id: Optional[str]
    """Tax ID for the customer (e.g.

    VAT number). Requires billing_address with country.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from dodopayments import DodoPayments, AsyncDodoPayments
from dodopayments._utils import parse_datetime
from dodopayments.pagination import SyncDefaultPageNumberPagination, AsyncDefaultPageNumberPagination
from dodopayments.types.credit_entitlements import (
    CreditLedgerEntry,
    CustomerCreditBalance,
    BalanceListGrantsResponse,
    BalanceCreateLedgerEntryResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestBalances:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_retrieve(self, client: DodoPayments) -> None:
        balance = client.credit_entitlements.balances.retrieve(
            customer_id="customer_id",
            credit_entitlement_id="credit_entitlement_id",
        )
        assert_matches_type(CustomerCreditBalance, balance, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: DodoPayments) -> None:
        response = client.credit_entitlements.balances.with_raw_response.retrieve(
            customer_id="customer_id",
            credit_entitlement_id="credit_entitlement_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        balance = response.parse()
        assert_matches_type(CustomerCreditBalance, balance, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: DodoPayments) -> None:
        with client.credit_entitlements.balances.with_streaming_response.retrieve(
            customer_id="customer_id",
            credit_entitlement_id="credit_entitlement_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            balance = response.parse()
            assert_matches_type(CustomerCreditBalance, balance, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: DodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `credit_entitlement_id` but received ''"):
            client.credit_entitlements.balances.with_raw_response.retrieve(
                customer_id="customer_id",
                credit_entitlement_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `customer_id` but received ''"):
            client.credit_entitlements.balances.with_raw_response.retrieve(
                customer_id="",
                credit_entitlement_id="credit_entitlement_id",
            )

    @parametrize
    def test_method_list(self, client: DodoPayments) -> None:
        balance = client.credit_entitlements.balances.list(
            credit_entitlement_id="credit_entitlement_id",
        )
        assert_matches_type(SyncDefaultPageNumberPagination[CustomerCreditBalance], balance, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: DodoPayments) -> None:
        balance = client.credit_entitlements.balances.list(
            credit_entitlement_id="credit_entitlement_id",
            customer_id="customer_id",
            page_number=0,
            page_size=0,
        )
        assert_matches_type(SyncDefaultPageNumberPagination[CustomerCreditBalance], balance, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: DodoPayments) -> None:
        response = client.credit_entitlements.balances.with_raw_response.list(
            credit_entitlement_id="credit_entitlement_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        balance = response.parse()
        assert_matches_type(SyncDefaultPageNumberPagination[CustomerCreditBalance], balance, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: DodoPayments) -> None:
        with client.credit_entitlements.balances.with_streaming_response.list(
            credit_entitlement_id="credit_entitlement_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            balance = response.parse()
            assert_matches_type(SyncDefaultPageNumberPagination[CustomerCreditBalance], balance, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list(self, client: DodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `credit_entitlement_id` but received ''"):
            client.credit_entitlements.balances.with_raw_response.list(
                credit_entitlement_id="",
            )

    @parametrize
    def test_method_create_ledger_entry(self, client: DodoPayments) -> None:
        balance = client.credit_entitlements.balances.create_ledger_entry(
            customer_id="customer_id",
            credit_entitlement_id="credit_entitlement_id",
            amount="amount",
            entry_type="credit",
        )
        assert_matches_type(BalanceCreateLedgerEntryResponse, balance, path=["response"])

    @parametrize
    def test_method_create_ledger_entry_with_all_params(self, client: DodoPayments) -> None:
        balance = client.credit_entitlements.balances.create_ledger_entry(
            customer_id="customer_id",
            credit_entitlement_id="credit_entitlement_id",
            amount="amount",
            entry_type="credit",
            expires_at=parse_datetime("2019-12-27T18:11:19.117Z"),
            idempotency_key="idempotency_key",
            metadata={"foo": "string"},
            reason="reason",
        )
        assert_matches_type(BalanceCreateLedgerEntryResponse, balance, path=["response"])

    @parametrize
    def test_raw_response_create_ledger_entry(self, client: DodoPayments) -> None:
        response = client.credit_entitlements.balances.with_raw_response.create_ledger_entry(
            customer_id="customer_id",
            credit_entitlement_id="credit_entitlement_id",
            amount="amount",
            entry_type="credit",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        balance = response.parse()
        assert_matches_type(BalanceCreateLedgerEntryResponse, balance, path=["response"])

    @parametrize
    def test_streaming_response_create_ledger_entry(self, client: DodoPayments) -> None:
        with client.credit_entitlements.balances.with_streaming_response.create_ledger_entry(
            customer_id="customer_id",
            credit_entitlement_id="credit_entitlement_id",
            amount="amount",
            entry_type="credit",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            balance = response.parse()
            assert_matches_type(BalanceCreateLedgerEntryResponse, balance, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_create_ledger_entry(self, client: DodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `credit_entitlement_id` but received ''"):
            client.credit_entitlements.balances.with_raw_response.create_ledger_entry(
                customer_id="customer_id",
                credit_entitlement_id="",
                amount="amount",
                entry_type="credit",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `customer_id` but received ''"):
            client.credit_entitlements.balances.with_raw_response.create_ledger_entry(
                customer_id="",
                credit_entitlement_id="credit_entitlement_id",
                amount="amount",
                entry_type="credit",
            )

    @parametrize
    def test_method_list_grants(self, client: DodoPayments) -> None:
        balance = client.credit_entitlements.balances.list_grants(
            customer_id="customer_id",
            credit_entitlement_id="credit_entitlement_id",
        )
        assert_matches_type(SyncDefaultPageNumberPagination[BalanceListGrantsResponse], balance, path=["response"])

    @parametrize
    def test_method_list_grants_with_all_params(self, client: DodoPayments) -> None:
        balance = client.credit_entitlements.balances.list_grants(
            customer_id="customer_id",
            credit_entitlement_id="credit_entitlement_id",
            page_number=0,
            page_size=0,
            status="active",
        )
        assert_matches_type(SyncDefaultPageNumberPagination[BalanceListGrantsResponse], balance, path=["response"])

    @parametrize
    def test_raw_response_list_grants(self, client: DodoPayments) -> None:
        response = client.credit_entitlements.balances.with_raw_response.list_grants(
            customer_id="customer_id",
            credit_entitlement_id="credit_entitlement_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        balance = response.parse()
        assert_matches_type(SyncDefaultPageNumberPagination[BalanceListGrantsResponse], balance, path=["response"])

    @parametrize
    def test_streaming_response_list_grants(self, client: DodoPayments) -> None:
        with client.credit_entitlements.balances.with_streaming_response.list_grants(
            customer_id="customer_id",
            credit_entitlement_id="credit_entitlement_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            balance = response.parse()
            assert_matches_type(SyncDefaultPageNumberPagination[BalanceListGrantsResponse], balance, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list_grants(self, client: DodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `credit_entitlement_id` but received ''"):
            client.credit_entitlements.balances.with_raw_response.list_grants(
                customer_id="customer_id",
                credit_entitlement_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `customer_id` but received ''"):
            client.credit_entitlements.balances.with_raw_response.list_grants(
                customer_id="",
                credit_entitlement_id="credit_entitlement_id",
            )

    @parametrize
    def test_method_list_ledger(self, client: DodoPayments) -> None:
        balance = client.credit_entitlements.balances.list_ledger(
            customer_id="customer_id",
            credit_entitlement_id="credit_entitlement_id",
        )
        assert_matches_type(SyncDefaultPageNumberPagination[CreditLedgerEntry], balance, path=["response"])

    @parametrize
    def test_method_list_ledger_with_all_params(self, client: DodoPayments) -> None:
        balance = client.credit_entitlements.balances.list_ledger(
            customer_id="customer_id",
            credit_entitlement_id="credit_entitlement_id",
            end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            page_number=0,
            page_size=0,
            start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            transaction_type="transaction_type",
        )
        assert_matches_type(SyncDefaultPageNumberPagination[CreditLedgerEntry], balance, path=["response"])

    @parametrize
    def test_raw_response_list_ledger(self, client: DodoPayments) -> None:
        response = client.credit_entitlements.balances.with_raw_response.list_ledger(
            customer_id="customer_id",
            credit_entitlement_id="credit_entitlement_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        balance = response.parse()
        assert_matches_type(SyncDefaultPageNumberPagination[CreditLedgerEntry], balance, path=["response"])

    @parametrize
    def test_streaming_response_list_ledger(self, client: DodoPayments) -> None:
        with client.credit_entitlements.balances.with_streaming_response.list_ledger(
            customer_id="customer_id",
            credit_entitlement_id="credit_entitlement_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            balance = response.parse()
            assert_matches_type(SyncDefaultPageNumberPagination[CreditLedgerEntry], balance, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list_ledger(self, client: DodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `credit_entitlement_id` but received ''"):
            client.credit_entitlements.balances.with_raw_response.list_ledger(
                customer_id="customer_id",
                credit_entitlement_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `customer_id` but received ''"):
            client.credit_entitlements.balances.with_raw_response.list_ledger(
                customer_id="",
                credit_entitlement_id="credit_entitlement_id",
            )


class TestAsyncBalances:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncDodoPayments) -> None:
        balance = await async_client.credit_entitlements.balances.retrieve(
            customer_id="customer_id",
            credit_entitlement_id="credit_entitlement_id",
        )
        assert_matches_type(CustomerCreditBalance, balance, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncDodoPayments) -> None:
        response = await async_client.credit_entitlements.balances.with_raw_response.retrieve(
            customer_id="customer_id",
            credit_entitlement_id="credit_entitlement_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        balance = await response.parse()
        assert_matches_type(CustomerCreditBalance, balance, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncDodoPayments) -> None:
        async with async_client.credit_entitlements.balances.with_streaming_response.retrieve(
            customer_id="customer_id",
            credit_entitlement_id="credit_entitlement_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            balance = await response.parse()
            assert_matches_type(CustomerCreditBalance, balance, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncDodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `credit_entitlement_id` but received ''"):
            await async_client.credit_entitlements.balances.with_raw_response.retrieve(
                customer_id="customer_id",
                credit_entitlement_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `customer_id` but received ''"):
            await async_client.credit_entitlements.balances.with_raw_response.retrieve(
                customer_id="",
                credit_entitlement_id="credit_entitlement_id",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncDodoPayments) -> None:
        balance = await async_client.credit_entitlements.balances.list(
            credit_entitlement_id="credit_entitlement_id",
        )
        assert_matches_type(AsyncDefaultPageNumberPagination[CustomerCreditBalance], balance, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncDodoPayments) -> None:
        balance = await async_client.credit_entitlements.balances.list(
            credit_entitlement_id="credit_entitlement_id",
            customer_id="customer_id",
            page_number=0,
            page_size=0,
        )
        assert_matches_type(AsyncDefaultPageNumberPagination[CustomerCreditBalance], balance, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncDodoPayments) -> None:
        response = await async_client.credit_entitlements.balances.with_raw_response.list(
            credit_entitlement_id="credit_entitlement_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        balance = await response.parse()
        assert_matches_type(AsyncDefaultPageNumberPagination[CustomerCreditBalance], balance, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncDodoPayments) -> None:
        async with async_client.credit_entitlements.balances.with_streaming_response.list(
            credit_entitlement_id="credit_entitlement_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            balance = await response.parse()
            assert_matches_type(AsyncDefaultPageNumberPagination[CustomerCreditBalance], balance, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list(self, async_client: AsyncDodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `credit_entitlement_id` but received ''"):
            await async_client.credit_entitlements.balances.with_raw_response.list(
                credit_entitlement_id="",
            )

    @parametrize
    async def test_method_create_ledger_entry(self, async_client: AsyncDodoPayments) -> None:
        balance = await async_client.credit_entitlements.balances.create_ledger_entry(
            customer_id="customer_id",
            credit_entitlement_id="credit_entitlement_id",
            amount="amount",
            entry_type="credit",
        )
        assert_matches_type(BalanceCreateLedgerEntryResponse, balance, path=["response"])

    @parametrize
    async def test_method_create_ledger_entry_with_all_params(self, async_client: AsyncDodoPayments) -> None:
        balance = await async_client.credit_entitlements.balances.create_ledger_entry(
            customer_id="customer_id",
            credit_entitlement_id="credit_entitlement_id",
            amount="amount",
            entry_type="credit",
            expires_at=parse_datetime("2019-12-27T18:11:19.117Z"),
            idempotency_key="idempotency_key",
            metadata={"foo": "string"},
            reason="reason",
        )
        assert_matches_type(BalanceCreateLedgerEntryResponse, balance, path=["response"])

    @parametrize
    async def test_raw_response_create_ledger_entry(self, async_client: AsyncDodoPayments) -> None:
        response = await async_client.credit_entitlements.balances.with_raw_response.create_ledger_entry(
            customer_id="customer_id",
            credit_entitlement_id="credit_entitlement_id",
            amount="amount",
            entry_type="credit",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        balance = await response.parse()
        assert_matches_type(BalanceCreateLedgerEntryResponse, balance, path=["response"])

    @parametrize
    async def test_streaming_response_create_ledger_entry(self, async_client: AsyncDodoPayments) -> None:
        async with async_client.credit_entitlements.balances.with_streaming_response.create_ledger_entry(
            customer_id="customer_id",
            credit_entitlement_id="credit_entitlement_id",
            amount="amount",
            entry_type="credit",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            balance = await response.parse()
            assert_matches_type(BalanceCreateLedgerEntryResponse, balance, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_create_ledger_entry(self, async_client: AsyncDodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `credit_entitlement_id` but received ''"):
            await async_client.credit_entitlements.balances.with_raw_response.create_ledger_entry(
                customer_id="customer_id",
                credit_entitlement_id="",
                amount="amount",
                entry_type="credit",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `customer_id` but received ''"):
            await async_client.credit_entitlements.balances.with_raw_response.create_ledger_entry(
                customer_id="",
                credit_entitlement_id="credit_entitlement_id",
                amount="amount",
                entry_type="credit",
            )

    @parametrize
    async def test_method_list_grants(self, async_client: AsyncDodoPayments) -> None:
        balance = await async_client.credit_entitlements.balances.list_grants(
            customer_id="customer_id",
            credit_entitlement_id="credit_entitlement_id",
        )
        assert_matches_type(AsyncDefaultPageNumberPagination[BalanceListGrantsResponse], balance, path=["response"])

    @parametrize
    async def test_method_list_grants_with_all_params(self, async_client: AsyncDodoPayments) -> None:
        balance = await async_client.credit_entitlements.balances.list_grants(
            customer_id="customer_id",
            credit_entitlement_id="credit_entitlement_id",
            page_number=0,
            page_size=0,
            status="active",
        )
        assert_matches_type(AsyncDefaultPageNumberPagination[BalanceListGrantsResponse], balance, path=["response"])

    @parametrize
    async def test_raw_response_list_grants(self, async_client: AsyncDodoPayments) -> None:
        response = await async_client.credit_entitlements.balances.with_raw_response.list_grants(
            customer_id="customer_id",
            credit_entitlement_id="credit_entitlement_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        balance = await response.parse()
        assert_matches_type(AsyncDefaultPageNumberPagination[BalanceListGrantsResponse], balance, path=["response"])

    @parametrize
    async def test_streaming_response_list_grants(self, async_client: AsyncDodoPayments) -> None:
        async with async_client.credit_entitlements.balances.with_streaming_response.list_grants(
            customer_id="customer_id",
            credit_entitlement_id="credit_entitlement_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            balance = await response.parse()
            assert_matches_type(AsyncDefaultPageNumberPagination[BalanceListGrantsResponse], balance, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list_grants(self, async_client: AsyncDodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `credit_entitlement_id` but received ''"):
            await async_client.credit_entitlements.balances.with_raw_response.list_grants(
                customer_id="customer_id",
                credit_entitlement_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `customer_id` but received ''"):
            await async_client.credit_entitlements.balances.with_raw_response.list_grants(
                customer_id="",
                credit_entitlement_id="credit_entitlement_id",
            )

    @parametrize
    async def test_method_list_ledger(self, async_client: AsyncDodoPayments) -> None:
        balance = await async_client.credit_entitlements.balances.list_ledger(
            customer_id="customer_id",
            credit_entitlement_id="credit_entitlement_id",
        )
        assert_matches_type(AsyncDefaultPageNumberPagination[CreditLedgerEntry], balance, path=["response"])

    @parametrize
    async def test_method_list_ledger_with_all_params(self, async_client: AsyncDodoPayments) -> None:
        balance = await async_client.credit_entitlements.balances.list_ledger(
            customer_id="customer_id",
            credit_entitlement_id="credit_entitlement_id",
            end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            page_number=0,
            page_size=0,
            start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            transaction_type="transaction_type",
        )
        assert_matches_type(AsyncDefaultPageNumberPagination[CreditLedgerEntry], balance, path=["response"])

    @parametrize
    async def test_raw_response_list_ledger(self, async_client: AsyncDodoPayments) -> None:
        response = await async_client.credit_entitlements.balances.with_raw_response.list_ledger(
            customer_id="customer_id",
            credit_entitlement_id="credit_entitlement_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        balance = await response.parse()
        assert_matches_type(AsyncDefaultPageNumberPagination[CreditLedgerEntry], balance, path=["response"])

    @parametrize
    async def test_streaming_response_list_ledger(self, async_client: AsyncDodoPayments) -> None:
        async with async_client.credit_entitlements.balances.with_streaming_response.list_ledger(
            customer_id="customer_id",
            credit_entitlement_id="credit_entitlement_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            balance = await response.parse()
            assert_matches_type(AsyncDefaultPageNumberPagination[CreditLedgerEntry], balance, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list_ledger(self, async_client: AsyncDodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `credit_entitlement_id` but received ''"):
            await async_client.credit_entitlements.balances.with_raw_response.list_ledger(
                customer_id="customer_id",
                credit_entitlement_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `customer_id` but received ''"):
            await async_client.credit_entitlements.balances.with_raw_response.list_ledger(
                customer_id="",
                credit_entitlement_id="credit_entitlement_id",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["FilterOperator"]

FilterOperator: TypeAlias = Literal[
    "equals",
    "not_equals",
    "greater_than",
    "greater_than_or_equals",
    "less_than",
    "less_than_or_equals",
    "contains",
    "does_not_contain",
]
